@@ -1,10 +1,12 @@
+'use strict';
+
 var fs = require('fs');
 var path = require('path');
 var shell = require('shelljs');
 var semver = require('semver');
 var _ = require('lodash');
 
-var currentPackage, previousVersions, cdnVersion;
+var currentPackage, previousVersions, cdnVersion, gitRepoInfo;
 
 
 /**
@@ -154,14 +156,14 @@ var getCdnVersion = function() {
       }
       return cdnVersion;
     }, null);
-}
+};
 
 /**
  * Get the unstable snapshot version
  * @return {SemVer} The snapshot version
  */
 var getSnapshotVersion = function() {
-  version = _(previousVersions)
+  var version = _(previousVersions)
     .filter(function(tag) {
       return semver.satisfies(tag, currentPackage.branchVersion);
     })
