@@ -1,6 +1,6 @@
+/* global LocationHashbangUrl: false, LocationHtml5Url: false */
 'use strict';
 
-
 describe('$location', function() {
   var url;
 
@@ -14,6 +14,7 @@ describe('$location', function() {
 
 
   describe('File Protocol', function () {
+    /* global urlParsingNode: true */
     var urlParsingNodePlaceholder;
 
     beforeEach(inject(function ($sniffer) {
@@ -117,6 +118,15 @@ describe('$location', function() {
     });
 
 
+    it('search() should remove multiple parameters', function() {
+      url.search({one: 1, two: true});
+      expect(url.search()).toEqual({one: 1, two: true});
+      url.search({one: null, two: null});
+      expect(url.search()).toEqual({});
+      expect(url.absUrl()).toBe('http://www.domain.com:9877/path/b#hash');
+    });
+
+
     it('search() should handle multiple value', function() {
       url.search('a&b');
       expect(url.search()).toEqual({a: true, b: true});
@@ -221,7 +231,7 @@ describe('$location', function() {
 
     it('should set path to forward-slash when empty', function() {
       url = new LocationHtml5Url('http://server/');
-      url.$$parse('http://server/')
+      url.$$parse('http://server/');
       expect(url.path()).toBe('/');
       expect(url.absUrl()).toBe('http://server/');
     });
@@ -313,6 +323,18 @@ describe('$location', function() {
         expect(url.search()).toEqual({'i j': '<>#'});
         expect(url.hash()).toBe('x <>#');
       });
+
+      it('should decode pluses as spaces in urls', function() {
+        url = new LocationHtml5Url('http://host.com/');
+        url.$$parse('http://host.com/?a+b=c+d');
+        expect(url.search()).toEqual({'a b':'c d'});
+      });
+
+      it('should retain pluses when setting search queries', function() {
+        url.search({'a+b':'c+d'});
+        expect(url.search()).toEqual({'a+b':'c+d'});
+      });
+
     });
   });
 
@@ -437,28 +459,28 @@ describe('$location', function() {
 
       it('should return decoded characters for search specified with setter', function() {
         var locationUrl = new LocationHtml5Url('http://host.com/');
-        locationUrl.$$parse('http://host.com/')
+        locationUrl.$$parse('http://host.com/');
         locationUrl.search('q', '1/2 3');
         expect(locationUrl.search()).toEqual({'q': '1/2 3'});
       });
 
       it('should return an array for duplicate params', function() {
         var locationUrl = new LocationHtml5Url('http://host.com');
-        locationUrl.$$parse('http://host.com')
+        locationUrl.$$parse('http://host.com');
         locationUrl.search('q', ['1/2 3','4/5 6']);
         expect(locationUrl.search()).toEqual({'q': ['1/2 3','4/5 6']});
       });
 
       it('should encode an array correctly from search and add to url', function() {
         var locationUrl = new LocationHtml5Url('http://host.com');
-        locationUrl.$$parse('http://host.com')
+        locationUrl.$$parse('http://host.com');
         locationUrl.search({'q': ['1/2 3','4/5 6']});
         expect(locationUrl.absUrl()).toEqual('http://host.com?q=1%2F2%203&q=4%2F5%206');
       });
 
       it('should rewrite params when specifing a single param in search', function() {
         var locationUrl = new LocationHtml5Url('http://host.com');
-        locationUrl.$$parse('http://host.com')
+        locationUrl.$$parse('http://host.com');
         locationUrl.search({'q': '1/2 3'});
         expect(locationUrl.absUrl()).toEqual('http://host.com?q=1%2F2%203');
         locationUrl.search({'q': '4/5 6'});
@@ -747,7 +769,7 @@ describe('$location', function() {
   });
 
   describe('PATH_MATCH', function() {
-
+    /* global PATH_MATCH: false */
     it('should parse just path', function() {
       var match = PATH_MATCH.exec('/path');
       expect(match[1]).toBe('/path');
@@ -779,18 +801,31 @@ describe('$location', function() {
 
     var root, link, originalBrowser, lastEventPreventDefault;
 
-    function configureService(linkHref, html5Mode, supportHist, attrs, content) {
+    function configureService(linkHref, html5Mode, supportHist, relLink, attrs, content) {
+      if (typeof relLink !== "boolean") {
+        content = attrs;
+        attrs = relLink;
+        relLink = false;
+      }
       module(function($provide, $locationProvider) {
         attrs = attrs ? ' ' + attrs + ' ' : '';
 
         // fake the base behavior
-        if (linkHref[0] == '/') {
-          linkHref = 'http://host.com' + linkHref;
-        } else if(!linkHref.match(/:\/\//)) {
-          linkHref = 'http://host.com/base/' + linkHref;
+        if (typeof linkHref === 'string') {
+          if (!relLink) {
+            if (linkHref[0] == '/') {
+              linkHref = 'http://host.com' + linkHref;
+            } else if(!linkHref.match(/:\/\//)) {
+              linkHref = 'http://host.com/base/' + linkHref;
+            }
+          }
         }
 
-        link = jqLite('<a href="' + linkHref + '"' + attrs + '>' + content + '</a>')[0];
+        if (linkHref) {
+          link = jqLite('<a href="' + linkHref + '"' + attrs + '>' + content + '</a>')[0];
+        } else {
+          link = jqLite('<a ' + attrs + '>' + content + '</a>')[0];
+        }
 
         $provide.value('$sniffer', {history: supportHist});
         $locationProvider.html5Mode(html5Mode);
@@ -905,6 +940,34 @@ describe('$location', function() {
     });
 
 
+    // Regression (gh-7721)
+    it('should not throw when clicking anchor with no href attribute when history enabled on old browser', function() {
+      configureService(null, true, false);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser) {
+          browserTrigger(link, 'click');
+          expectNoRewrite($browser);
+        }
+      );
+    });
+
+
+    it('should produce relative paths correctly when $location.path() is "/" when history enabled on old browser', function() {
+      configureService('partial1', true, false, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser, $location) {
+          $location.path('/');
+          browserTrigger(link, 'click');
+          expectRewriteTo($browser, 'http://host.com/base/index.html#!/partial1');
+        }
+      );
+    });
+
+
     it('should rewrite abs link to hashbang url when history enabled on old browser', function() {
       configureService('/base/link?a#b', true, false);
       inject(
@@ -957,6 +1020,32 @@ describe('$location', function() {
     });
 
 
+    it('should not rewrite links with `javascript:` URI', function() {
+      configureService(' jAvAsCrIpT:throw new Error("Boom!")', true, true, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser) {
+          browserTrigger(link, 'click');
+          expectNoRewrite($browser);
+        }
+      );
+    });
+
+
+    it('should not rewrite links with `mailto:` URI', function() {
+      configureService(' mAiLtO:foo@bar.com', true, true, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser) {
+          browserTrigger(link, 'click');
+          expectNoRewrite($browser);
+        }
+      );
+    });
+
+
     it('should rewrite full url links to same domain and base path', function() {
       configureService('http://host.com/base/new', true);
       inject(
@@ -1067,8 +1156,53 @@ describe('$location', function() {
     });
 
 
+    it('should rewrite relative links relative to current path when history disabled', function() {
+      configureService('link', true, false, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser, $location) {
+          $location.path('/some');
+          browserTrigger(link, 'click');
+          expectRewriteTo($browser, 'http://host.com/base/index.html#!/some/link');
+        }
+      );
+    });
+
+
+    it('should replace current path when link begins with "/" and history disabled', function() {
+      configureService('/link', true, false, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser, $location) {
+          $location.path('/some');
+          browserTrigger(link, 'click');
+          expectRewriteTo($browser, 'http://host.com/base/index.html#!/link');
+        }
+      );
+    });
+
+
+    it('should replace current hash fragment when link begins with "#" history disabled', function() {
+      configureService('#link', true, false, true);
+      inject(
+        initBrowser(),
+        initLocation(),
+        function($browser, $location) {
+          // Initialize browser URL
+          $location.path('/some');
+          $location.hash('foo');
+          browserTrigger(link, 'click');
+          expect($location.hash()).toBe('link');
+          expectRewriteTo($browser, 'http://host.com/base/index.html#!/some#link');
+        }
+      );
+    });
+
+
     // don't run next tests on IE<9, as browserTrigger does not simulate pressed keys
-    if (!(msie < 9)) {
+    if (!msie || msie >= 9) {
 
       it('should not rewrite when clicked with ctrl pressed', function() {
         configureService('/a?b=c', true, true);
@@ -1102,10 +1236,10 @@ describe('$location', function() {
       module(function() {
         return function($browser) {
           window.location.hash = 'someHash';
-          base = window.location.href
+          base = window.location.href;
           $browser.url(base);
           base = base.split('#')[0];
-        }
+        };
       });
       inject(function($rootScope, $compile, $browser, $rootElement, $document, $location) {
         // we need to do this otherwise we can't simulate events
@@ -1137,7 +1271,7 @@ describe('$location', function() {
           $browser.url(base = window.location.href);
           base = base.split('#')[0];
           $locationProvider.hashPrefix('!');
-        }
+        };
       });
       inject(function($rootScope, $compile, $browser, $rootElement, $document, $location) {
         // we need to do this otherwise we can't simulate events
@@ -1170,7 +1304,7 @@ describe('$location', function() {
         });
         return function($browser) {
           $browser.url(base = 'http://server/');
-        }
+        };
       });
       inject(function($location) {
         // make IE happy
@@ -1200,7 +1334,7 @@ describe('$location', function() {
         });
         return function($browser) {
           $browser.url(base = 'http://server/');
-        }
+        };
       });
       inject(function($rootScope, $compile, $browser, $rootElement, $document, $location) {
         // make IE happy
@@ -1241,7 +1375,7 @@ describe('$location', function() {
           $browser.url(base = window.location.href);
           base = base.split('#')[0];
           $locationProvider.hashPrefix('!');
-        }
+        };
       });
       inject(function($rootScope, $compile, $browser, $rootElement, $document, $location) {
         // we need to do this otherwise we can't simulate events
@@ -1296,7 +1430,7 @@ describe('$location', function() {
         event.preventDefault();
       });
       $rootScope.$on('$locationChangeSuccess', function(event, newUrl, oldUrl) {
-        throw Error('location should have been canceled');
+        throw new Error('location should have been canceled');
       });
 
       expect($location.url()).toEqual('');
@@ -1346,7 +1480,7 @@ describe('$location', function() {
           $rootElement.html('<a href="http://server/#/somePath">link</a>');
           $compile($rootElement)($rootScope);
           jqLite(document.body).append($rootElement);
-        }
+        };
       });
 
       inject(function($location, $rootScope, $browser, $rootElement) {
@@ -1379,7 +1513,7 @@ describe('$location', function() {
           $rootElement.html('<a href="http://server/somePath">link</a>');
           $compile($rootElement)($rootScope);
           jqLite(document.body).append($rootElement);
-        }
+        };
       });
 
       inject(function($location, $rootScope, $browser, $rootElement) {
@@ -1455,6 +1589,7 @@ describe('$location', function() {
     var location;
 
     it('should rewrite URL', function() {
+      /* jshint scripturl: true */
       location = new LocationHashbangUrl('http://server/pre/', '#');
 
       expect(location.$$rewrite('http://other')).toEqual(undefined);
@@ -1466,7 +1601,7 @@ describe('$location', function() {
     it("should not set hash if one was not originally specified", function() {
       location = new LocationHashbangUrl('http://server/pre/index.html', '#');
 
-      location.$$parse('http://server/pre/index.html')
+      location.$$parse('http://server/pre/index.html');
       expect(location.url()).toBe('');
       expect(location.absUrl()).toBe('http://server/pre/index.html');
     });
@@ -1474,7 +1609,7 @@ describe('$location', function() {
     it("should parse hash if one was specified", function() {
       location = new LocationHashbangUrl('http://server/pre/index.html', '#');
 
-      location.$$parse('http://server/pre/index.html#/foo/bar')
+      location.$$parse('http://server/pre/index.html#/foo/bar');
       expect(location.url()).toBe('/foo/bar');
       expect(location.absUrl()).toBe('http://server/pre/index.html#/foo/bar');
     });
@@ -1483,14 +1618,25 @@ describe('$location', function() {
     it("should prefix hash url with / if one was originally missing", function() {
       location = new LocationHashbangUrl('http://server/pre/index.html', '#');
 
-      location.$$parse('http://server/pre/index.html#not-starting-with-slash')
+      location.$$parse('http://server/pre/index.html#not-starting-with-slash');
       expect(location.url()).toBe('/not-starting-with-slash');
       expect(location.absUrl()).toBe('http://server/pre/index.html#/not-starting-with-slash');
     });
+
+
+    it('should not strip stuff from path just because it looks like Windows drive when it\'s not',
+        function() {
+      location = new LocationHashbangUrl('http://server/pre/index.html', '#');
+
+      location.$$parse('http://server/pre/index.html#http%3A%2F%2Fexample.com%2F');
+      expect(location.url()).toBe('/http://example.com/');
+      expect(location.absUrl()).toBe('http://server/pre/index.html#/http://example.com/');
+    });
   });
 
 
   describe('LocationHashbangInHtml5Url', function() {
+    /* global LocationHashbangInHtml5Url: false */
     var location, locationIndex;
 
     beforeEach(function() {
